
/* nxprob41.c						*/
/* program for nxlcau41 option 't'			*/
/* calculate probabilities related to evolution		*/
/* Harold V. McIntosh, 10 August 1987			*/
/* 4 September 1987 - tetrahedral stereopair		*/
/* 30 December 1987 - contoured stereopair layers	*/
/* 3 April 1989 - n, N for totalistic statistics	*/
/* 15 September 1989 - Entropy calculations		*/
/* 30 April 1990 - subset indices, equivalence indices	*/
/* 10 July 1994 - adapted to NeXT			*/

/* references:							*/
/*								*/
/* W. John Wilbur, David J. Lipman and Shihab A. Shamma		*/
/* On the prediction of local patterns in cellular automata	*/
/* Physica 19D 397-410 (1986)					*/ 
/*								*/
/* Howard A. Gutowitz, Jonathan D. Victor and Bruce W. Knight	*/
/* Local structure theory for cellular automata			*/
/* Physica 28D 18-48 (1987)					*/

/*	Copyright (C) 1987	*/
/*	Copyright (C) 1988	*/
/*	Copyright (C) 1989	*/
/*	Copyright (C) 1990	*/
/*	Copyright (C) 1994	*/
/*	Harold V. McIntosh	*/

# include <math.h>
# include "nxlcau41.h"

extern char   ascrule[KK][KK][KK];		/* ASCII transition values  */
extern char   arr1[AL], arr2[AL];		/* line of cells            */
extern int    binrule[KK][KK][KK];		/* binary transition values */
extern double wmul[KK], wmvl[KK];		/* left mass point          */
extern double wmur[KK], wmvr[KK];		/* right mass point         */
extern double blpr[KK];				/* probability from blocks  */

# define BER	    0	/* row (from bottom) for Bernstein p.	*/
# define BEC	  205	/* column for Bernstein Polynomial	*/
# define BROW	   14   /* row for bar charts		*/
# define EROW	   23   /* row for evolution synopsis	*/
# define CORG 	    0	/* x-origin  of Bernstein plot  */

/* edit the probability screen	*/

edtri() {char c; int i;
  videomode(COLGRAF);
  videopalette(YELREGR);

  videocursor(0,1,33);  videoputc('2',3);
  videocursor(0,13,23); videoputc('0',3);
  videocursor(0,13,39); videoputc('1',3);

  tbase();
  edges(50,3);

  while (0<1) {
  videocursor(0,0,0);  hscrrul();
  videocursor(0,0,36); videoputc('?',2);
  c=kbdin();
  if (c == '\015') break;
  videocursor(0,0,38); videoputc(c,2);
  videocursor(0,0,36); videoputc(' ',0);
  switch (c) {
    case '+': videopalette(WHCYMAG); break;
    case '-': videopalette(YELREGR); break;
    case 'a': nblclr(); asfreq(3); break;
    case 'b': bpanel(); break;
    case 'B': tbase(); break;
    case 'E': tedge(); break;
    case 'e': pevolve(); break;
    case 'g': for (i=0, asctobin(); i<50; i++, onegen(AL)) lifreq(2); break;
    case 'G': for (i=0, asctobin(); i<200; i++, onegen(AL)) lifreq(1); break;
    case 'h': edges(80,3); break;
    case 'k': videohegrid(10,0); break;
    case 'l': lpanel(); break;
    case 'm': nblclr(); moncar(1,1); break;
    case 'M': nblclr(); moncar(50,2); break;
    case 'n': nblclr(); neisum(8*BROW,AL,48); break;
    case 'N': nblclr(); nbrsum(8*BROW,AL,48); break;
    case 'p': elayer(40,0); break;
    case 'q': elayer(40,1); break;
//    case 'Q': eqrndx(); break;
    case 'r': elayer(40,2); break;
    case 's': elayer(40,3); break;
//    case 'S': subndx(); break;
    case 't': hpdiff(12,8); break;
    case 'u': elayer(72,8); break;
    case 'v': olayer(72,8); break;
    case 'U': elayer(42,8); break;
    case 'V': olayer(42,8); break;
    case 'x': ranprul(); videocursor(0,0,0); hscrrul(); break;
    case 'X': ranrul(); videocursor(0,0,0); hscrrul(); break;
    case '1': nblclr(); oneblfreq(8*BROW,300,56); break;
    case '2': nblclr(); twoblfreq(8*BROW,300,56); break;
    case '3': nblclr(); thrblfreq(8*BROW,300,56); break;
    case '4': nblclr(); foublfreq(8*BROW,300,56); break;
    case '5': nblclr(); fivblfreq(8*BROW,300,56); break;
    case 'z': nblclr(); break;
    case '/': videomode(COLGRAF); videopalette(YELREGR); break;
    case '!': nblclr(); pamenu(); break;
    case '?': nblclr(); trmenu(); break;
    default: break;
    } /* end switch */
  }   /* end while  */
  videopalette(WHCYMAG);
  videomode(T80X25);
}     /* end edtri  */

/* show probability curves                */
/* e  = evolution (0) or iteration (1)    */
/* g  = generation (0=frame and diagonal) */
/* eo = even (0) or odd (1) layer         */
/* eb = base (1) or edge (0) placement    */
void procur(e,g,eo,eb) int e, g, eo, eb; {
// printf("procur has e = %2d, g = %2d, eo= %2d, eb = %2d\n",e,g,eo,eb);
  if (eb==1) tbase(); else tedge();
  switch (e) {
    case 0: switch (g) {
//      case 0: gfram(CORG,0,100); gdiag(CORG,0,100); break;
      case 1: if (eo==0) elayer(72,8); else olayer(72,8); break;
      case 2: if (eo==0) edlayer(72,8); else odlayer(72,8); break;
      case 3: if (eo==0) etlayer(72,8); else otlayer(72,8); break;
//      case 4: pdiff5(100); break;
//      case 5: pdiff6(100); break;
      default: break;} break;
    case 1: switch (g) {
//      case 0: gfram(CORG,0,100); gdiag(CORG,0,100); break;
    case 2: elayer(72,8); break;
//    case 2: pidiff(100); break;
//    case 3: pjdiff(100); break;
        default: break;} break;
    default: break;}
}

/* show n-block probabilities */
void nblcur(n) int n; {switch (n) {
    case '1': oneblfreq(0,300,48); break;
    case '2': twoblfreq(0,300,48); break;
    case '3': thrblfreq(0,300,48); break;
    case '4': foublfreq(0,300,48); break;
    case '5': fivblfreq(0,300,48); break;
//    case '6': sixblfreq(0,300,48); break;
//    case '7': svnblfreq(0,300,48); break;
    default: break;}
}

/* show option menu */

trmenu() {
  videocursor(0,BROW,0);
  printf("a     - a priori estimates, entropy\n");
  printf("m,g,G - sample evolution\n");
  printf("pqrs  - prob of states\n");
  printf("t,u,v - sq, odd-even\n");
  printf("12345 - n-block bar charts\n");
  printf("n,N   - totalistic statistics\n");
  printf("e     - 16 lines evolution\n");
  printf("z/?!  - clear panel, screen; show menu\n");
  printf("<cr>  - exit\n");
}

/* show parameter menu */

pamenu() {
  videocursor(0,BROW,0);
  printf("B     - tetrahedron on face           \n");
  printf("E     - tetrahedron on edge           \n");
  printf("h     - draw edges                    \n");
  printf("k     - draw grid                     \n");
  printf("      -                               \n");
  printf("+-    - change color quad             \n");
  printf("      -                               \n");
  printf("z/?!  - clear panel, screen; show menu\n");
  printf("<cr>  - exit\n");
}

/* show sixteen lines of evolution on screen */

pevolve() {int i, j;
  videoscroll(EROW,0,EROW+1,40,0,0);
  asctobin();
  for (j=8*EROW; j<8*EROW+16; j++) {
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    }
}

/* Clear a space for the n-block frequencies */
nblclr() {videoscroll(BROW,0,BROW+8,40,0,0);}

/* locations of the weighted masses */
/* for tetrahedron sitting on base  */

tbase() {
  wmul[0]=160.0; wmvl[0]=100.0;
  wmul[1]=310.0; wmvl[1]=100.0;
  wmul[2]=240.0; wmvl[2]=195.0;
  wmul[3]=250.0; wmvl[3]=157.0;

  wmur[0]= 10.0; wmvr[0]=100.0;
  wmur[1]=160.0; wmvr[1]=100.0;
  wmur[2]= 90.0; wmvr[2]=195.0;
  wmur[3]= 80.0; wmvr[3]=157.0;
}

/* locations of the weighted masses */
/* for tetrahedron standing on edge */

tedge() {
  wmul[0]=160.0; wmvl[0]=157.0;
  wmul[1]=310.0; wmvl[1]=157.0;
  wmul[2]=240.0; wmvl[2]=195.0;
  wmul[3]=250.0; wmvl[3]=100.0;

  wmur[0]= 10.0; wmvr[0]=157.0;
  wmur[1]=160.0; wmvr[1]=157.0;
  wmur[2]= 90.0; wmvr[2]=195.0;
  wmur[3]= 80.0; wmvr[3]=100.0;
}

/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(199-BER-j,BEC+i,l);}

/* pdot(i,j,l) - place polkadot on Bernstein screen at (i,j) in color l */
pdot(i,j,l) int i, j, l; {videodot(199-BER-j,BEC+i,l+i%2);}

/* plot a single point on a triangular grid */

videotrdot(u,v,x,y,z,l) double x, y, z; int u, v, l; {double s;
s=199.0/(x+y+z); x=x*s; y=y*s; z=z*s;
videodot(u-(int)(0.433*z),v+(int)(0.250*(y+y+z)),l);
}

/* plot a single point on a tetrahedral grid */

videohedot(a,b,p,l) double p[KK]; int a, b, l; {double s, u, v, w;
s=199.0/(p[0]+p[1]+p[2]+p[3]);
u=(p[1]+p[3])*s; v=p[2]*s; w=(p[1]-p[3])*s;
videodot(a-(int)(0.433*p[1]*s),b-50+(int)(0.250*(u+u+v)),l);
videodot(a-(int)(0.433*p[1]*s),b+50+(int)(0.250*(w+w+v)),l);
}

/* plot a stereo point on a triangular grid */
/* (u,v) screen location lower left corner  */
/* (x,y,z) triangular coordinates	    */
/* t       height			    */
/* l       color			    */

videostdot(u,v,x,y,z,t,l) double x, y, z, t; int u, v, l; {double s;
s=199.0/(x+y+z); x=x*s; y=y*s; z=z*s;
videodot(u-(int)(0.433*z),v+(int)(0.250*(y+y+z+t)),l);
}

/* set up a triangular gridwork */
/* (u,v) = corner of triangle	*/
/* n = number of rulings	*/
/* l = color of lines		*/

videotrgrid(u,v,n,l) int u, v, n, l; {
int i, j;
double a, b, c;

if(n==0) return;
for (i=0; i<=n;  i++) {
for (j=0; i+j<=n; j++) {
  a=((double)(i)/(double)(n));
  b=((double)(j)/(double)(n));
  c=1.0-a-b;
  videotrdot(u,v,a,b,c,l);
  }}
}

/* display state frequencies in arr1 as a dot in Bezier diagram */

lifreq(l) int l; {int i, stat[KK]; double staf[KK], s;
// printf("l from lifreq = %3d\n",l);
s=1.0/((double)(AL));
for (i=0; i<KK; i++) stat[i]=0;
for (i=0; i<AL; i++) (stat[arr1[i]])++;
for (i=0; i<KK; i++) staf[i]=s*((double)(stat[i]));
videomassdot(staf,l);
}

/* plot a single point as a weighted mass center */

videomassdot(x,l) double *x; int l; {double ul, vl, ur, vr; int i;
// if (l>=KK) return;
for (i=0, ul=0.0; i<KK; i++) ul+=x[i]*wmul[i];
for (i=0, vl=0.0; i<KK; i++) vl+=x[i]*wmvl[i];
for (i=0, ur=0.0; i<KK; i++) ur+=x[i]*wmur[i];
for (i=0, vr=0.0; i<KK; i++) vr+=x[i]*wmvr[i];
videodot(199-(int)vl,(int)ul,l);
videodot(199-(int)vr,(int)ur,l);
}

/* plot a point in 4 triangles - face projections of a tetrahedron */

video4dots(x,l) int l; double x[KK]; {double u[KK-1];
u[0]=x[0]; u[1]=x[1]; u[2]=x[2]; videotrdot(99,200,u,l);
u[0]=x[1]; u[1]=x[2]; u[2]=x[3]; videotrdot(199,0,u,l);
u[0]=x[2]; u[1]=x[3]; u[2]=x[0]; videotrdot(149,100,u,l);
u[0]=x[3]; u[1]=x[0]; u[2]=x[1]; videotrdot(199,200,u,l);
}

/* set up a triangular gridwork of edge n in color l */

videohegrid(n,l) int n, l; {
int i0, i1, i2;
double x[KK], nn;

if (n==0) return;
nn=1.0/((double)(n));
for (i0=0, x[0]=0.0; i0<=n; i0++, x[0]+=nn) {
for (i1=0, x[1]=0.0; i0+i1<=n; i1++, x[1]+=nn) {
for (i2=0, x[2]=0.0; i0+i1+i2<=n; i2++, x[2]+=nn) {
  x[3]=1.0-x[0]-x[1]-x[2];
  videomassdot(x,l);
  }}}
}

/* show the skeleton framework of the tetrahedron */
/* n = points per edge				  */
/* l = color of edge				  */

edges(n,l) int n, l; {int i; double x[KK], nn;
if (n==0) return;
nn=1.0/((double)(n));
for (i=0, uvec(x,0); i<=n; i++, x[0]-=nn, x[1]+=nn) videomassdot(x,l);
for (i=0, uvec(x,0); i<=n; i++, x[0]-=nn, x[2]+=nn) videomassdot(x,l);
for (i=0, uvec(x,0); i<=n; i++, x[0]-=nn, x[3]+=nn) videomassdot(x,l);
for (i=0, uvec(x,1); i<=n; i++, x[1]-=nn, x[2]+=nn) videomassdot(x,l);
for (i=0, uvec(x,1); i<=n; i++, x[1]-=nn, x[3]+=nn) videomassdot(x,l);
for (i=0, uvec(x,2); i<=n; i++, x[2]-=nn, x[3]+=nn) videomassdot(x,l);
}

/* unit vector */

uvec(x,i) double *x; int i; {int j;
for (j=0; j<KK; j++) x[j]=0.0;
x[i]=1.0;
}

/* homogeneous vector */

hvec(x) double *x; {int i; double s;
s=1.0/((double)(KK));
for (i=0; i<KK; i++) x[i]=s;
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

elayer(n,l) int n, l; {int m, nn;
  m=n/4; nn=4*m;
  pdiff(3*m,nn,l);
  pdiff(2*m,nn,l);
  pdiff(1*m,nn,l);
  pdiff(0*m,nn,l);
  edges(nn,3);
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

olayer(n,l) int n, l; {int m, nn;
  m=n/8; nn=8*m;
  pdiff(7*m,nn,l);
  pdiff(5*m,nn,l);
  pdiff(3*m,nn,l);
  pdiff(1*m,nn,l);
  edges(nn,3);
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

edlayer(n,l) int n, l; {int m, nn;
  m=n/4; nn=4*m;
  dpdiff(3*m,nn,l);
  dpdiff(2*m,nn,l);
  dpdiff(1*m,nn,l);
  dpdiff(0*m,nn,l);
  edges(nn,3);
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

odlayer(n,l) int n, l; {int m, nn;
  m=n/8; nn=8*m;
  dpdiff(7*m,nn,l);
  dpdiff(5*m,nn,l);
  dpdiff(3*m,nn,l);
  dpdiff(1*m,nn,l);
  edges(nn,3);
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

etlayer(n,l) int n, l; {int m, nn;
  m=n/4; nn=4*m;
  tpdiff(3*m,nn,l);
  tpdiff(2*m,nn,l);
  tpdiff(1*m,nn,l);
  tpdiff(0*m,nn,l);
  edges(nn,3);
}

/* layered contour map in stereoscopic tetrahedron */
/* n = length of edge  */
/* l = function option */

otlayer(n,l) int n, l; {int m, nn;
  m=n/8; nn=8*m;
  tpdiff(7*m,nn,l);
  tpdiff(5*m,nn,l);
  tpdiff(3*m,nn,l);
  tpdiff(1*m,nn,l);
  edges(nn,3);
}

/* ------------ */
/* ------------- */

/* options 'm', 'M'.				*/
/* "Monte Carlo" determination of probabilities */
/* n = mumber of generations			*/
/* l = color of dot				*/

moncar(n,l) int n, l; {
int i, j, k, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

nblclr();
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
  videomassdot(bf,l);
  }
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
videocursor(0,BROW,0);
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
printf("lambda       = %5.3f\n",1.0-bf[0]);
printf("entropy      = %5.3f\n",-t*s);
printf("pair entropy = %5.3f\n",-0.5*tt*s);
printf("frequency    = ");
for (i=0; i<KK; i++) printf("%5.3f ",a[i]);
printf("\n");
if (n>1) {
  printf("deviation   =  ");
  for (i=0; i<KK; i++) printf("%5.3f ",d[i]);
  printf("\n");
  }
printf("pair freq    = ");
for (i=0; i<KK; i++) printf("%5.3f ",bbf[0][i]);
printf("\n");
for (j=1; j<KK; j++) {
  printf("               ");
  for (i=0; i<KK; i++) printf("%5.3f ",bbf[j][i]);
  printf("\n");
  }
}

/* option 'n' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

neisum(ll,mm,nn) int ll, mm, nn; {
int ii, i, k, m, n, l, b[TS];
double em, en, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  onesum(mm,TS,b);
  for (i=0; i<TS; i++) bf[i]=((double)(b[i]));
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  m=0; videodot(ll+ii,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i%2);
    }
  videodot(ll+ii,m++,3);
  k++;
  if (kbdst()) {kbdin(); break;}
  }  /* end for-ii */
  en=((double)(k));
  m=0; videodot(ll+nn,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bft[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(ll+nn,m++,i%2+1);
    }
  videodot(ll+nn,m++,3);
for (i=0; i<TS; i++) bft[i]/=en*em;
videocursor(0,BROW+7,0);
for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* option 'n' (general neighborhood) */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

void gnlsum(ll,mm,nn) int ll, mm, nn; {
int    i0, i1, i2;
int    ii, k, m, n, l, b[KK][KK][KK];
double em, en, f, bf[KK][KK][KK], bft[KK][KK][KK];

asctobin();
em=((double)(mm));
// f=1.0;
f=115.0/((double)(AL));
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bft[i0][i1][i2]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  gnrsum(mm,b);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bf[i0][i1][i2]=((double)(b[i0][i1][i2]));
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bft[i0][i1][i2]+=bf[i0][i1][i2];
  m=0; videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) {
    n=(int)(f*bf[i0][i1][i2]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i2%2+1);
    }
  videodot(ll+ii,m++,3);
  k++;
  }  /* end for-ii */
  en=((double)(k));
//  m=0; videodot(ll+nn,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) {
    n=(int)(0.1*(bft[i0][i1][i2]/en));
//    if (n>0) for (l=0; l<n; l++) videodot(ll+nn,m++,i0%2+1);
    }
//  videodot(ll+nn,m++,3);
// for (i=0; i<TS; i++) bft[i]/=en*em;
// videocursor(0,BROW+7,0);
// for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* option 'N' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

nbrsum(ll,mm,nn) int ll, mm, nn; {
int ii, jj, i, k, m, n, l, b[TS];
double em, en, kk, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
en=((double)(nn));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (jj=0; jj<nn; jj++) {
  for (i=0; i<TS; i++) bf[i]=0.0;
  for (ii=0; ii<nn; ii++) {
    onegen(mm);
    onesum(mm,TS,b);
    for (i=0; i<TS; i++) bf[i]+=((double)(b[i]));
    }  /* end for-ii */
  m=0; videodot(ll+jj,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(ll+jj,m++,i%2+1);
    }
  videodot(ll+jj,m++,3);
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  k++;
  if (kbdst()) {kbdin(); break;}
  } /* end for-jj */
kk=((double)(k));
videodot(ll+jj,m++,3);
for (i=0; i<TS; i++) bft[i]/=kk*en*em;
videocursor(0,BROW+7,0);
for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* gather totalistic neighborhood sums for one generation */
/* mm = number of cells */
/* b[cc]  = sum counts  */

onesum(mm,cc,b) int mm, cc, *b; {int i, j, k;
  for (i=0; i<cc; i++) b[i]=0;
  k=arr1[mm-1]+arr1[0]+arr1[1]; b[k]++;
  for (i=1; i<mm-2; i++) {k=0; for (j=i-1; j<i+2; j++) k+=arr1[j]; b[k]++;}
  k=arr1[mm-2]+arr1[mm-1]+arr1[0]; b[k]++;
}

/* gather general neighborhood sums for one generation */
/* mm = number of cells         */
/* b[KK][KK][KK]  = sum counts  */
void gnrsum(mm,b) int mm, b[KK][KK][KK]; {int i0, i1, i2, i;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	b[i0][i1][i2]=0;
  b[arr1[mm-1]][arr1[0]][arr1[1]]++;
  for (i=1; i<mm-2; i++) b[arr1[i-1]][arr1[i]][arr1[i+1]]++;
  b[arr1[mm-2]][arr1[mm-1]][arr1[0]]++;
}

/* make a grid for the fixpoint graph  */
/* n     = dimension of grid           */
ggrid(n) int n; {int i, j;
for (i=0; i<=n; i+=10) for (j=0; j<=n; j+=10) bdot(i,j,3);}

/* put a diagonal in the fixpoint graph */
gdiag(n) int n; {int i; for (i=0; 2*i<n; i++) bdot(2*i,2*i,3);}

/* plot self-consistent probability graph */
bpanel()  {ggrid(100); gdiag(100); bgraf(100);}

/* plot self-consistent graph in Langton's parameter */
lpanel()  {ggrid(100); gdiag(100); lgraf(100);}

/* graph Bernstein polynomial */
bgraf(n) int n; {
int    i;
double bern(), el, em, en, p, q, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  q=el*(1.0-p); t[1]=t[2]=t[3]=q;
  t[0]=p; pdot(i,(int)(en*bern(0,t)),1);
  t[0]=q; t[1]=p; bdot(i,(int)(en*bern(1,t)),1);
  t[1]=q; t[2]=p; bdot(i,(int)(en*bern(2,t)),2);
  t[2]=q; t[3]=p; bdot(i,(int)(en*bern(3,t)),3);
  }
}

/* graph Langton's parameter */
lgraf(n) int n; {int i;
double bern(), el, em, en, p, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  t[0]=p; t[1]=t[2]=t[3]=el*(1.0-p);
  pdot(n-i,n-(int)(en*bern(0,t)),1);
  }
}

/* graph the probability differential for state l over a triangle */
pdiff(i0,n,l) int i0, n, l; {
int    i1, i2;
double p[KK], en, bern(), sqsu();
if (n==0) return;
if (i0>n) return;
en=1.0/((double)(n));
p[0]=((double)(i0))*en;
for (i1=0, p[1]=0.0; i0+i1<=n; i1++, p[1]+=en) {
for (i2=0, p[2]=0.0; i0+i1+i2<=n; i2++, p[2]+=en) {
  p[3]=1.0-p[0]-p[1]-p[2];
  switch(l) {
    case 0: videomassdot(p,psign(bern(0,p)-p[0])); break;
    case 1: videomassdot(p,psign(bern(1,p)-p[1])); break;
    case 2: videomassdot(p,psign(bern(2,p)-p[2])); break;
    case 3: videomassdot(p,psign(bern(3,p)-p[3])); break;
    case 4: videomassdot(p,lico(bern(0,p),i1+i2)); break;
    case 5: videomassdot(p,lico(bern(1,p),i2)); break;
    case 6: videomassdot(p,lico(bern(2,p),i1)); break;
    case 7: videomassdot(p,lico(bern(3,p),i1)); break;
    case 8: videomassdot(p,sqco(sqsu(p),i1+i2)); break;
    default: break;
    }
//   if (kbdst()) {kbdin(); return;}
  }}
}

/* graph the probability differential for state l over a triangle */
dpdiff(i0,n,l) int i0, n, l; {
int    i1, i2;
double p[KK], en, dbern(), dsqsu();
if (n==0) return;
if (i0>n) return;
en=1.0/((double)(n));
p[0]=((double)(i0))*en;
for (i1=0, p[1]=0.0; i0+i1<=n; i1++, p[1]+=en) {
for (i2=0, p[2]=0.0; i0+i1+i2<=n; i2++, p[2]+=en) {
  p[3]=1.0-p[0]-p[1]-p[2];
  switch(l) {
    case 0: videomassdot(p,psign(dbern(0,p)-p[0])); break;
    case 1: videomassdot(p,psign(dbern(1,p)-p[1])); break;
    case 2: videomassdot(p,psign(dbern(2,p)-p[2])); break;
    case 3: videomassdot(p,psign(dbern(3,p)-p[3])); break;
    case 4: videomassdot(p,lico(dbern(0,p),i1+i2)); break;
    case 5: videomassdot(p,lico(dbern(1,p),i2)); break;
    case 6: videomassdot(p,lico(dbern(2,p),i1)); break;
    case 7: videomassdot(p,lico(dbern(3,p),i1)); break;
    case 8: videomassdot(p,sqco(dsqsu(p),i1+i2)); break;
    default: break;
    }
//   if (kbdst()) {kbdin(); return;}
  }}
}

/* graph the probability differential for state l over a triangle */
tpdiff(i0,n,l) int i0, n, l; {
int    i1, i2;
double p[KK], en, tbern(), tsqsu();
if (n==0) return;
if (i0>n) return;
en=1.0/((double)(n));
p[0]=((double)(i0))*en;
for (i1=0, p[1]=0.0; i0+i1<=n; i1++, p[1]+=en) {
for (i2=0, p[2]=0.0; i0+i1+i2<=n; i2++, p[2]+=en) {
  p[3]=1.0-p[0]-p[1]-p[2];
  switch(l) {
    case 0: videomassdot(p,psign(tbern(0,p)-p[0])); break;
    case 1: videomassdot(p,psign(tbern(1,p)-p[1])); break;
    case 2: videomassdot(p,psign(tbern(2,p)-p[2])); break;
    case 3: videomassdot(p,psign(tbern(3,p)-p[3])); break;
    case 4: videomassdot(p,lico(tbern(0,p),i1+i2)); break;
    case 5: videomassdot(p,lico(tbern(1,p),i2)); break;
    case 6: videomassdot(p,lico(tbern(2,p),i1)); break;
    case 7: videomassdot(p,lico(tbern(3,p),i1)); break;
    case 8: videomassdot(p,sqco(tsqsu(p),i1+i2)); break;
    default: break;
    }
//   if (kbdst()) {kbdin(); return;}
  }}
}

/* graph the probability differential for state l over a tetrahedron	*/
/* n = number of points per edge					*/
/* l = color of dot							*/

hpdiff(n,l) int n, l; {
int    i, j, k;
double bern(), x[KK], nn;

if (n==0) return;
nn=1.0/((double)(n));
for (i=0, x[0]=0.0; i<=n; i++, x[0]+=nn) {
for (j=0, x[1]=0.0; i+j<=n; j++, x[1]+=nn) {
for (k=0, x[2]=0.0; i+j+k<=n; k++, x[2]+=nn) {
  x[3]=1.0-x[0]-x[1]-x[2];
  switch(l) {
    case 0: videomassdot(x,psign(bern(0,x)-x[0])); break;
    case 1: videomassdot(x,psign(bern(1,x)-x[1])); break;
    case 2: videomassdot(x,psign(bern(2,x)-x[2])); break;
    case 3: videomassdot(x,psign(bern(3,x)-x[3])); break;
    case 4: videomassdot(x,lico(bern(0,x),i+j)); break;
    case 5: videomassdot(x,lico(bern(1,x),j)); break;
    case 6: videomassdot(x,lico(bern(2,x),i)); break;
    case 7: videomassdot(x,lico(bern(3,x),i)); break;
    case 8: videomassdot(x,sqco(sqsu(x),i+j)); break;
    default: break;
    }
//   if (kbdst()) {kbdin(); return;}
  }}}
}

pidiff(n,l,f) int n, l, f; {
int    i, j, k, m, sqco();
double a[KK], b[KK], c[KK], en, t, bern(), sqsu();
double u, v, w, x;
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++)
for (j=0; i+j<=n; j++)
for (k=0; i+j+k<=n; k++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=((double)(k))*en;
  a[3]=1.0-a[0]-a[1]-a[2];
  for (m=0; m<KK; m++) b[m]=a[m];
  for (k=0; k<f; k++) {
    for (m=0; m<KK; m++) c[m]=bern(m,b);
    for (m=0; m<KK; m++) b[m]=c[m];
    }
u=b[0]-a[0];
v=b[1]-a[1];
w=b[2]-a[2];
x=b[3]-a[3];
t= (u*u+v*v+w*w+x*x);
  videomassdot(a,sqco(t,i));
//  if (kbdst()) {kbdin(); return;}
  }
}

/* choose a color for the contour level based on sign: red, yellow, green */
psign(x) double x; {if (x>0.01) return 1; if (x<-0.01) return 2; return 3;}

/* generate squaresum compatible contour values */
int sqco(t,i) int i; double t; {int l;
if (t<0.0075) l=0; else {
if (t<0.0750) {if (i%2) l=0; else l=3;} else {
if (t<0.1250) l=3; else {
if (t<0.2500) {if (i%2) l=3; else l=2;} else {
if (t<0.5000) l=2; else {
if (t<0.7500) {if (i%2) l=2; else l=1;} else {
if (t<1.0000) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* generate linear contour values */
int lico (t,i) int i; double t; {int l;
if (t<0.1) l=0; else {
if (t<0.2) {if (i%2) l=0; else l=3;} else {
if (t<0.3) l=3; else {
if (t<0.4) {if (i%2) l=3; else l=2;} else {
if (t<0.5) l=2; else {
if (t<0.6) {if (i%2) l=2; else l=1;} else {
if (t<0.7) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* present the frequencies of ascrule in color l */
/* percentages evolving into each value		 */
/* percentages shifting - adds to more than 100% */
/* entropy and Langton's lambda = % nonquiescent */

asfreq(l) int l; {
int    i, j, i0, i1, i2;
int    stat[KK], stal, stac, star;		/* statistic counts */
double staf[KK], t, pp;

pp=1.0/((double)(KK*KK*KK));
stal=0; stac=0; star=0;
for (i=0; i<KK; i++) stat[i]=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
  j=ascrule[i0][i1][i2]-'0';
  stat[j]++;
  if(j==i0) star++;
  if(j==i1) stac++;
  if(j==i2) stal++;
  }}}
videocursor(0,BROW,0);
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
for (i=0; i<KK; i++) printf("%d freq - %5.2f\n",i,staf[i]);
printf("\n");
printf("left   - %5.2f\n",((double)stal)*pp);
printf("still  - %5.2f\n",((double)stac)*pp);
printf("right  - %5.2f\n",((double)star)*pp);
videomassdot(staf,l);
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
videocursor(0,BROW,20);
printf("lambda:   %5.3f",1.0-staf[0]);
videocursor(0,BROW+1,20);
printf("entropy:  %5.3f",-t/log((double)KK));
videocursor(0,BROW,0);
}

/* end nxprob41.c */
